<#
.SYNOPSIS
	Works like a virtualenvwrapper (http://www.doughellmann.com/projects/virtualenvwrapper/)
	Author: Dmitry Kozlov

.DESCRIPTION
	virtualenv- is a PowerShell Cmdlet for creating, switching and deleting virtual
	environments in a visual name.

	- Organizes all of your virtual environments in one place
	  see $virtualenvs variable. Folder should be created manually.
	- Activate, Deactivate (Enter) and Delete (Delete) environments from FAR menu

	KEYS AND ACTIONS in Menu

	[Enter]
	Activates/Deactivates selected environment

	[Delete]
	Deletes selected environment

	[CtrlEnter]
	Goes to site-packages directory of selected environment


.EXAMPLE:
	# Show menu
	>: virtualenv-

	# Create new/activate existing environment
	>: virtualenv- <name>

	# Create new environment with extra options
	>: virtualenv- <name> --unzip-setuptools --no-site-packages
#>

param (
	[string]
	$environ,

	[parameter(ValueFromRemainingArguments=$true)]
	$virtualenv_args
)

$virtualenvs = 'c:\virtualenvs'

function activate($env)
{
	$name = Split-Path $env -Leaf
	Write-Host Activating $name

	$command = Join-Path $env Scripts\activate.bat

	cmd /c "$command > nul 2>&1 && set" | .{process{
		if ($_ -match '^([^=]+)=(.*)') {
			[System.Environment]::SetEnvironmentVariable($matches[1], $matches[2])
		}
	}}
}

function deactivate
{
	$name = Split-Path $env:VIRTUAL_ENV -Leaf
	Write-Host Deactivating $name

	if($env:_OLD_VIRTUAL_PROMPT) {
		$env:PROMPT = $env:_OLD_VIRTUAL_PROMPT
		Remove-Item env:_OLD_VIRTUAL_PROMPT
	}

	if($env:_OLD_VIRTUAL_PYTHONHOME) {
		$env:PYTHONHOME = $env:_OLD_VIRTUAL_PYTHONHOME
		Remove-Item env:_OLD_VIRTUAL_PYTHONHOME
	}

	if($env:_OLD_VIRTUAL_PATH) {
		$env:PATH = $env:_OLD_VIRTUAL_PATH
		Remove-Item env:_OLD_VIRTUAL_PATH
	}
	if($env:VIRTUAL_ENV) {
		Remove-Item env:VIRTUAL_ENV
	}
}

$active_env = $env:VIRTUAL_ENV

if ($environ) {
	$path = Join-Path $virtualenvs $environ
	if(Test-Path $path) {
		activate $path
	}
	else
	{
		virtualenv $virtualenv_args $path
	}
	return
}

$menu = $Far.CreateMenu()
$menu.Title = 'virtualenvs'
$menu.AutoAssignHotkeys = $true
$menu.ShowAmpersands = $true
$menu.WrapCursor = $true
$menu.BreakKeys.Add([FarNet.VKeyCode]::Delete)
if ($Far.Window.Kind -eq 'Panels') {
	$menu.BreakKeys.Add([FarNet.VKeyCode]::Enter -bor [FarNet.VKeyMode]::Ctrl)
}


dir -LiteralPath $virtualenvs | where { $_.PSIsContainer } | .{process{
	$item = New-FarItem $_.Name -Data $_

	if ($active_env -eq $_.FullName) {
		$menu.Selected = $menu.Items.Count
		$item.Checked = $true
	}

	$menu.Items.Add($item)
}}

$null = $menu.Show()

$$ = $menu.SelectedData

if (!$$) { return }

if ($menu.breakKey -eq ([FarNet.VKeyCode]::Delete)) {
	$choice = Show-FarMessage "Do you wish to delete $$" -Buttons 'YesNo' -Caption 'Delete'
	if ($choice -eq 0) {
		if ($active_env -eq $$.FullName) {
			deactivate
		}
		Remove-Item -Recurse -Force $$.FullName
	}
	return
}

if ($menu.breakKey -eq ([FarNet.VKeyCode]::Enter -bor [FarNet.VKeyMode]::Ctrl)) {
	$path = Join-Path $$.FullName Lib\site-packages\
	Write-Host $path
	$Far.Panel.GoToPath($path)
	return
}

if ($active_env -eq $$.FullName) {
	deactivate
}
else {
	activate $$.FullName
}
